# 青空文庫から持ってきた文章のワードクラウドを作るスクリプト

import zipfile
import os.path, urllib.request as req
import re
from janome.tokenizer import Tokenizer
from wordcloud import WordCloud

url = "https://www.aozora.gr.jp/cards/000119/files/624_ruby_5668.zip" #山月記
# url = "https://www.aozora.gr.jp/cards/000148/files/773_ruby_5968.zip" #こころ
# url = "https://www.aozora.gr.jp/cards/000074/files/424_ruby_19825.zip" #檸檬
local = url.split('/')[-1]
if not os.path.exists(local):
    req.urlretrieve(url, local)     # zipファイルをローカルにダウンロード

zip = zipfile.ZipFile(local, 'r')
for filename in zip.namelist():
    if filename.split('.')[-1] != 'txt' : continue
    outfile = filename.split('.')[0] + '.wc.png'
    txt = zip.open(filename, 'r').read().decode('cp932')
zip.close()

txt = re.split(r'\-{5,}', txt)[2]   # 本文の前を除去
txt = re.split(r'\r\n底本', txt)[0] # 本文の後を除去
txt = txt.strip()
txt = txt.replace('｜','')
txt = re.sub(r'《.+?》','', txt)    # ルビを除去
txt = re.sub(r'［＃.+?］','', txt)  # 各種説明を除去

t = Tokenizer()
freq = {}
lines = txt.split('\r\n')
for line in lines:
    tokens = t.tokenize(line)
    for token in tokens:
        w = token.base_form             # 単語の基本形を拾う
        if w == '*': w = token.surface
        ps = token.part_of_speech       # 品詞
        if w == 'Ｋ': ps = '名詞'       # 「こころ」ではＫを特別扱いしてみる
        if ps.split(',')[0] in ['名詞']:
            if w not in freq: freq[w] = 0
            freq[w] += 1

wc = WordCloud(width=1024,height=768,font_path='ipaexg.ttf',background_color='white')
wc.generate_from_frequencies(freq)
wc.to_file(outfile)
